/*
 * Mmap.cc -- Definitions.  Kinda crummy.
 *
 * devphil@users.sourceforge.net
 * $Id: Mmap.cc,v 1.1.1.1 2000/03/05 01:15:15 pedwards Exp $
*/


#include <Mmap.h>

#ifdef HAVE_MMAP
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdexcept>

#ifndef MAP_FAILED
#define MAP_FAILED  ((void*) -1)
#endif

#else
#error mmap workaround not enabled -- you must have mmap(2)!
#endif /* HAVE_MMAP */


namespace {
    void oopsies (const std::string& er)
    {
        Mmap::errortext = "Mmap error:  " + er;
        Mmap::errortext.append (":  ");
        Mmap::errortext.append (strerror(errno));
        if (Mmap::nothrow == false)
            throw std::runtime_error(Mmap::errortext.c_str());
    }
}  // end unnamed namespace


std::string Mmap::errortext;
bool        Mmap::nothrow = false;

Mmap::Mmap (std::string const& filename, int prot, int flags)
    : srcp (NULL),
      sz (0)
{
    struct stat  ticky;
    int          fd;
    int          openflags = O_RDWR;

    if ((prot & PROT_WRITE) != PROT_WRITE)  openflags = O_RDONLY;

    if ((fd = open (filename.c_str(), openflags)) < 0) {
        oopsies ("couldn't open(2) " + filename);
        return;
    }
    if (fstat (fd, &ticky) < 0) {
        oopsies ("couldn't fstat(2) " + filename);
        return;
    }
    if ((srcp =  mmap (0, ticky.st_size, prot, flags, fd, 0)) == MAP_FAILED)
    {
        oopsies ("couldn't mmap(2) " + filename);
        return;
    }
    sz = ticky.st_size;
    close (fd);
}


Mmap::~Mmap ()
{
    munmap (srcp, sz);
    srcp = NULL;
}


std::string Mmap::str () const
{
    return std::string (static_cast<char*>(srcp), sz);
}

